#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-


import re
from lxml import etree
try:
    from hashlib import md5
except ImportError:
    from md5 import md5



try:
    import uuid
except ImportError:
    SECRET_SALT = str(randint(0, 1000000))
else:
    SECRET_SALT = str(uuid.uuid4())


def _hash_text(s):
    return 'md5' + md5(SECRET_SALT + s).hexdigest()


def _hash_text_utf8(s):
    return 'md5' + md5(SECRET_SALT + s.encode("utf-8")).hexdigest()


class HashList(object):

    def __init__(self):
        self._hash_dictionary = {}

    def set_hash(self, text, parent):
        result_child_text = etree.tostring(parent)
        result_child = etree.XML(result_child_text)
        link_or_image = False
        link_text = ""
        if len(parent) != 0:
            for children in parent:
                link_or_image = False
                pre_link_re = re.compile('''[ ]''', re.X | re.M)
                pre_link = pre_link_re.sub("[ ]", etree.tostring(children))
                link_re = re.compile('''%s''' % pre_link, re.X | re.M)
                if children.tag == "link":
                    link_or_image = True
                    link_text = "[%s](%s \"%s\")" % (children.text,children.get('uri'), children.get('title'))
                if children.tag == "image":
                        link_or_image = True
                        link_text = "![%s](%s \"%s\")" % (children.get('alt'),children.get('uri'),children.get('title'))
                if link_or_image == True:
                    l_key = _hash_text(link_text)
                    child_text = link_re.sub(l_key, etree.tostring(result_child))
                    result_child = etree.XML(child_text)
                    self._hash_dictionary[l_key] = link_text
        return result_child

    def _hash_code_sub(self, match):
        text = match.group(1)
        code_key = _hash_text(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def set_e_hash(self, text, e_or_h_re):
        text = e_or_h_re.sub(self._hash_code_sub, text)
        return text

    def create_hash(self, text):
        code_key = _hash_text(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def create_hash_utf8(self, text):
        code_key = _hash_text_utf8(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def unhash(self, text):
        for key, sanitized in self._hash_dictionary.items():
            text = text.replace(key, sanitized)
        return text

    def un_hash(self, text):
        for hash_key in self._hash_dictionary.keys():
            hash_re = re.compile('''%s''' % hash_key, re.X | re.M)
            text = hash_re.sub(self._hash_dictionary[hash_key], text)
        return text

    def un_hash_with_tag(self, text, tag):
        for hash_key in self._hash_dictionary.keys():
            hash_re = re.compile('''%s''' % tag + hash_key, re.X | re.M)
            text = hash_re.sub(self._hash_dictionary[hash_key], text)
        return text

    def get_hash_list(self):
        return self._hash_dictionary

    def set_hash_item(self, key, item):
        self._hash_dictionary[key] = item
