#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-

import re


class Syntax(object):

    _hr_data = [
                ('*', re.compile(r"^[ ]{0,3}\*(.*?)$", re.M)),
                ('-', re.compile(r"^[ ]{0,3}\-(.*?)$", re.M)),
                ('_', re.compile(r"^[ ]{0,3}\_(.*?)$", re.M)),
                ]
    _syntax_index = ["java","c","c\+\+","objective-c","objectivec","javascript","python"]

    def __init__(self):
        self._re_index = []
        self._set_index()
    
    def _set_index(self):
        for index in self._syntax_index:
            test_re = re.compile(r'''^\s*%s\s*$''' % (index), re.M | re.X | re.IGNORECASE)
            self._re_index.append(test_re)
    
    def get_syntax(self,comment):
        for(syntax_re, index) in zip(self._re_index, self._syntax_index):
            m = syntax_re.match(comment)
            if m != None:
                text = syntax_re.sub(index, comment)
                return text
        return ""
    

    def do_line(self, text):
        hr = "\n<line" + self.empty_element_suffix + "\n"
        for ch, regex in self._hr_data:
            if ch in text:
                for m in reversed(list(regex.finditer(text))):
                    tail = m.group(1).rstrip()
                    if not tail.strip(ch + ' ') and tail.count("   ") == 0:
                        start, end = m.span()
                        text = text[:start] + hr + text[end:]
        return text
