#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-

__author__ = "Y.Ban"

import re
from lxml import etree
try:
    from hashlib import md5
except ImportError:
    from md5 import md5

try:
    import uuid
except ImportError:
    SECRET_SALT = str(randint(0, 1000000))
else:
    SECRET_SALT = str(uuid.uuid4())


def _hash_text(s):
    return 'md5' + md5(SECRET_SALT + s.encode("utf-8")).hexdigest()


def _hash_text_utf8(s):
    return 'md5' + md5(SECRET_SALT + s.encode("utf-8")).hexdigest()


def _convert_and(text):
    text = text.replace('&', '&gt;')
    return text


class HashList(object):
    """Hash table Class"""

    def __init__(self):
        self._hash_dictionary = {}

    def set_hash(self, text, parent):
        """Make a hash table."""
        result_child_text = etree.tostring(parent)
        result_child_text = result_child_text.replace("(", "&#40;")
        result_child_text = result_child_text.replace(")", "&#41;")
        result_child = etree.XML(result_child_text)
        link_or_image = False
        link_text = ""
        if len(parent) != 0:
            for children in parent:
                link_or_image = False
                prepre_link_re = re.compile('''[ ]''', re.X | re.M)
                prepre_link = prepre_link_re.sub("[ ]", etree.tostring(children))
                pre_link_re = re.compile('''.*</%s>''' % children.tag, re.X | re.M)
                if children.tag == "link":
                    link_re = re.compile('''<link.*?</link>''', re.X | re.M)
                elif children.tag == "image":
                    link_re = re.compile('''<image.*?/>''', re.X | re.M)
                if children.tag == "link":
                    link_or_image = True
                    link_text = "[%s](%s \"%s\")" % (children.text, children.get('uri'), children.get('title'))
                if children.tag == "image":
                    link_or_image = True
                    link_text = "![%s](%s \"%s\")" % (children.get('alt'), children.get('uri'), children.get('title'))
                if link_or_image is True:
                    l_key = _hash_text(link_text)
                    child_text = link_re.sub(l_key, etree.tostring(result_child))
                    result_child = etree.XML(child_text)
                    self._hash_dictionary[l_key] = link_text
        return result_child

    def _match_ampsharp(self, match):
        text = match.group(2)
        return "&" + text

    def _convert_ampsharp(self, text):
        text = self._convert_sixteen_to_ten(text)
        ampsharp_re = re.compile('''(&\#(\d+);)''', re.X | re.M)
        text = ampsharp_re.sub(self._match_ampsharp, text)
        return text

    def _match_sixten(self, match):
        text = match.group(2)
        return "&#" + str(int(text, 16)) + ";"

    def _convert_sixteen_to_ten(self, text):
        ampsharp_re = re.compile('''(&\#x(\w+);)''', re.X | re.M)
        text = ampsharp_re.sub(self._match_sixten, text)
        return text

    def _hash_code_sub(self, match):
        text = match.group(1)
        code_key = _hash_text(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def set_e_hash(self, text, e_or_h_re):
        """Make hash for equal and hypen."""
        text = e_or_h_re.sub(self._hash_code_sub, text)
        return text

    def create_hash(self, text):
        code_key = _hash_text(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def create_hash_utf8(self, text):
        code_key = _hash_text_utf8(text)
        self._hash_dictionary[code_key] = text
        return code_key

    def unhash(self, text):
        for key, sanitized in self._hash_dictionary.items():
            text = text.replace(key, sanitized)
        return text

    def un_hash(self, text):
        """Get text from a hash table."""
        for hash_key in self._hash_dictionary.keys():
            hash_re = re.compile('''%s''' % hash_key, re.X | re.M)
            if isinstance(text, unicode):
                texts = text
            else:
                texts = unicode(text, 'utf_8')
            text = hash_re.sub(self._hash_dictionary[hash_key], texts)
        return text

    def un_hash_with_tag(self, text, tag):
        for hash_key in self._hash_dictionary.keys():
            hash_re = re.compile('''%s''' % tag + hash_key, re.X | re.M)
            text = hash_re.sub(self._hash_dictionary[hash_key], text)
        return text

    def get_hash_list(self):
        return self._hash_dictionary

    def set_hash_item(self, key, item):
        self._hash_dictionary[key] = item
