#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-

__author__ = "Y.Ban"

import re
from lxml import etree
try:
    from hashlib import md5
except ImportError:
    from md5 import md5


class Paragraph(object):
    """Paragraph conversion Class"""
    def __init__(self):
        self._title_dictionary = {"part": 1, "chapter": 2, "section": 3, "subsection": 4, "subsubsection": 5, "subsubsubsection": 6}
        self._em_dictionary = {"emphasis": 1, "strong": 2, "heavy": 3}
        self._htmltag_list = ["div", "strike", "span", "big", "small", "ol", "ul", "a", "s"]

        self._add_escape_re = re.compile('''([\\\\\#\+\-\`\_\*])''' , re.X | re.M)

    def get_title_num(self, title):
        """Get a title type."""
        return self._title_dictionary.get(title, -1)

    def get_em_num(self, em):
        """Get a emphasis type"""
        return self._em_dictionary.get(em, 0)
    
    def is_html_tag(self, tag):
        return tag in self._htmltag_list

    def get_content(self, child, tag):
        """Get a content sandwiched by tags."""
        r = re.compile(tag)
        splitted = r.split(etree.tostring(child, encoding='utf-8'))
        text = splitted[1]
        return text

    def _e_sub(self, match):
        result = match.group(1)
        return "\\%s" % result

    def add_escape(self, child):
        """Add escape marks."""
        text = etree.tostring(child, encoding='utf-8')
        result = etree.XML(self.add_escape_text(text))
        return result
    
    def add_escape_text(self, text):
        """Add escape marks."""
        text = self._add_escape_re.sub(self._e_sub, text)
        text = text.replace("![", "!\[")
        text = text.replace("][", "]\[")
        text = text.replace("](", "]\(")
        return text

    def convert_entity(self, text):
        """Convert entity."""
        text = text.replace('&lt;', '<')
        text = text.replace('&gt;', '>')
        text = text.replace('&amp;', '&')
        return text
