#!/usr/bin/env python
# Copyright (C) 2013, Xcoo, Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*-

__author__ = "Y.Ban"

from lxml import etree
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
import os.path
import urllib2
import shutil
from hashhtml import HashHtml


class Writer(object):
    """Xml Converter Class"""

    def __init__(self, xmlpath, outputpath):
        self.buf = ""
        self._xmlpath = xmlpath
        self._output_path = outputpath
        self._basename = os.path.splitext(os.path.basename(outputpath))[0]
        self._hash_html = HashHtml()


    def _img_download(self, img_dir, url):
        try:
            img = urllib2.urlopen(url)
        except (urllib2.HTTPError, IOError):
            return None
        img_path = self._check_name(os.path.join(img_dir, os.path.basename(url)))
        img_name = 'images/' + os.path.basename(img_path)
        localfile = open(img_path, 'wb')
        localfile.write(img.read())
        img.close()
        localfile.close()
        return img_name

    def _check_name(self, file_name):
        if not os.path.exists(file_name):
            return file_name
        else:
            file_exist = True
            index = 0
            split = os.path.splitext(file_name)
            while file_exist:
                index += 1
                file_exist = os.path.exists(split[0] + '(%s)' % index + split[1])
            return split[0] + '(%s)' % index + split[1]

    def _packaging(self, root):
        image_list = root.xpath('//image')
        if os.path.dirname(self._output_path) == "":
            self._output_dir = '.'
        else:
            self._output_dir = os.path.dirname(self._output_path)
        output_dir = self._check_name(os.path.join(self._output_dir, self._basename))
        img_output_dir = output_dir + '/images'
        os.makedirs(img_output_dir)
        img_uri = []
        for img in image_list:
            if img.get('uri').startswith('file://localhost'):
                img.set('uri', img.get('uri').replace('file://localhost', ''))
            if img.get('uri').startswith('http://') or img.get('uri').startswith('https://'):
                download = self._img_download(img_output_dir, img.get('uri'))
                img_uri.append(img.get('uri'))
                if not download is None:
                    img.set('uri', download)
            else:
                if img.get('uri') in img_uri:
                    img_path = os.path.join(img_output_dir,  os.path.basename(img.get('uri')))
                else:
                    img_path = self._check_name(os.path.join(img_output_dir,  os.path.basename(img.get('uri'))))
                try:
                    shutil.copy(os.path.join(os.path.dirname(self._xmlpath), img.get('uri')), img_path)
                except:
                    shutil.copy(img.get('uri'), img_path)
                img_uri.append(img.get('uri'))
                img.set('uri', 'images/' + os.path.basename(img_path))
        self._output_path = os.path.join(output_dir, os.path.basename(self._output_path))
        return root

    def parse_xml(self, pack=False):
        """Parse xml for lxml."""
        encoding = "UTF-8"
        fp = open(self._xmlpath, encoding)
        self._buf = fp.read()
        fp.close()
        self._buf = unicode(self._buf, 'utf-8')
        # Turn block-level HTML blocks into hash entries
        self._buf = self._hash_html.hash_html_blocks(self._buf, raw=True)
        try:
            roots = etree.XML(self._buf)
        except:
            parser = etree.XMLParser(recover=True)
            roots = etree.fromstring(self._buf, parser=parser)
        if pack is True:
            roots = self._packaging(roots)
        return roots
    
    def unhash_html(self, text):
        text = self._hash_html.un_hash(text)
        return text

    def _save(self, output):
        fp = open(self._output_path, "w")
        fp.write(output)
        fp.close()

    def export(self, output_text):
        """Export to a file."""
        if isinstance(output_text, unicode):
            self._save(output_text.encode("utf-8"))
        else:
            self._save(output_text)
